//
//  PSPWareHelperController.h
//  PSPWareHelper
//
//  Created by Maksim Rogov on Sun Mar 06 2005.
//  Copyright (c) 2005 Nullriver, Inc. All rights reserved.

#import <Cocoa/Cocoa.h>

#import "PrefsController.h"
#import "PSPWSyncManager.h"
#import "PSFParser.h"
#import "NRLogger.h"

#define	PSPWHelperDidStartNotification @"PSPWHelperDidStartNotification"

@protocol PSPWareApplicationProtocol
	- (BOOL)isOpen;
	- (void)doAlert:(bycopy in NSString *)theTitle message:(bycopy in NSString *)theMessage style:(bycopy in NSAlertStyle)alertStyle;
	- (void)writeToLog:(bycopy in NSString *)message;
	- (void)startConversionAnimation;
	- (void)stopConversionAnimation;
	- (void)updateUserInterface;
	- (void)reloadMoviesView;
	- (void)updateSyncBytes;
	- (void)updateSyncBytesEstimate;
@end

@protocol PSPWareHelperProtocol
	- (void)setApplication:(byref in id)theApplication;
	- (oneway void)applicationWillQuit;

	- (oneway IBAction)ejectPSP:(bycopy in id)sender;
	- (oneway IBAction)openPSPWare:(bycopy in id)sender;
	- (oneway IBAction)syncNow:(bycopy in id)sender;

	- (bycopy NSString *)log;
	- (bycopy NSNumber *)isFirstSync;
	- (bycopy NSNumber *)pspIsMounted;
	- (bycopy NSNumber *)canSync;
	- (bycopy NSNumber *)isSyncing;
	- (bycopy NSNumber *)canCancelSync;
	- (bycopy NSNumber *)isIndeterminate;
	- (bycopy NSNumber *)syncBytesTotal;
	- (bycopy NSNumber *)syncBytes;
	- (bycopy NSString *)statusText;
	- (bycopy NSString *)statusSubText;
	- (void)setIsInteractive:(in BOOL)yorn;
	- (bycopy NSString *)percentConverted;

	- (oneway void)changeStartOnLogin;
	- (void)clearLog;

	- (bycopy NSNumber *)syncPercentEstimate;
	- (bycopy NSString *)syncBytesEstimate;
	- (oneway void)refreshSyncBytesEstimate;

	- (bycopy NSArray *)musicItems;
	- (oneway void)setSelectedMusicIdentifiers:(bycopy in NSArray *)selectedIdentifiers;

	- (bycopy NSArray *)photosItems;
	- (oneway void)setSelectedPhotosIdentifiers:(bycopy in NSArray *)selectedIdentifiers;

	- (bycopy NSArray *)bookmarksItems;
	- (oneway void)setSelectedBookmarksIdentifiers:(bycopy in NSArray *)selectedIdentifiers;


	- (bycopy NSArray *)moviesItems;
	- (oneway void)setSelectedMoviesIdentifiers:(bycopy in NSArray *)selectedIdentifiers;
	- (BOOL)isConvertingMovie;
	- (void)addMoviesFromArray:(bycopy in NSArray *)arrayOfMovies;
	- (void)removeMovieItem:(bycopy in NSDictionary *)movieItem;
	- (void)renameMovieItem:(bycopy in NSMutableDictionary *)movieItem newName:(bycopy in NSString *)newMovieName;

	- (bycopy NSArray *)backupItems;
	- (oneway void)setSelectedBackupIdentifiers:(bycopy in NSArray *)selectedIdentifiers;
	- (oneway void)logChanged:(bycopy in NSString *)newMessage;
@end

@interface PSPWareHelperController : NSObject <PSPWareHelperProtocol> {
    IBOutlet			id		menu;
	
	NRLogger			*		logger;
	NSStatusItem		*		statusItem;
	PSPWSyncManager		*		syncManager;
	
	NSTimer				*		syncAnimationTimer;
	unsigned					syncAnimationFrame;
	int							syncAnimationDirection;

	NSConnection		*		helperConnection;

	BOOL						isInteractive;
	NSString			*		statusText;
	NSString			*		statusSubText;

	NSString			*		percentConverted;
	NSMutableArray		*		moviesQueue;
	NSMutableArray		*		moviesFailed;
	NSTimer				*		movieQueueTimer;
	NSTask				*		convertTask;
	NSMutableString		*		movieLogBuffer;
	unsigned					totalFrames;
	//BOOL						isConvertingMovie;
	BOOL						movieQueueDidAbort;

	id							application;
}

// Accessors
- (NRLogger *)logger;

// Application Connection
- (void)setApplication:(id)theApplication;

// Interface
- (IBAction)ejectPSP:(id)sender;
- (IBAction)openPSPWare:(id)sender;
- (IBAction)syncNow:(id)sender;

// Accessors
- (NSString *)log;
- (NSNumber *)isFirstSync;
- (NSNumber *)pspIsMounted;
- (NSNumber *)canSync;
- (NSNumber *)isSyncing;
- (NSNumber *)canCancelSync;
- (NSNumber *)isIndeterminate;
- (NSNumber *)syncBytesTotal;
- (NSNumber *)syncBytes;
- (NSString *)statusText;
- (void)setStatusText:(NSString *)theText;
- (NSString *)statusSubText;
- (void)setStatusSubText:(NSString *)theText;
- (BOOL)isInteractive;
- (void)setIsInteractive:(BOOL)yorn;
- (NSString *)percentConverted;
- (void)setPercentConverted:(NSString *)theString;

// UI  Notifier
- (void)notifyUserInterface;

// Utility
- (void)doAlert:(NSString *)title message:(NSString *)message style:(NSAlertStyle)style;
- (NSString *)relativeDate:(NSDate *)obj;

// Sync Animation
- (void)syncAnimationPing;
- (void)startSyncAnimation;
- (void)stopSyncAnimation;

// Startup Items
- (void)changeStartOnLogin;
- (void)editStartupItems:(NSString*)path enabled:(BOOL)enabled hide:(BOOL)aValue;

// Music
- (NSArray *)musicItems;
- (void)setSelectedMusicIdentifiers:(NSArray *)selectedIdentifiers;

// Photos
- (NSArray *)photosItems;
- (void)setSelectedPhotosIdentifiers:(NSArray *)selectedIdentifiers;

// Bookmarks
- (NSArray *)bookmarksItems;
- (void)setSelectedBookmarksIdentifiers:(NSArray *)selectedIdentifiers;

// Movies
- (NSArray *)moviesItems;
- (void)setSelectedMoviesIdentifiers:(NSArray *)selectedIdentifiers;
- (BOOL)isConvertingMovie;
/* - (void)setIsConvertingMovie:(BOOL)yorn; */
- (void)addMoviesFromArray:(NSArray *)arrayOfMovies;
- (void)manageMovieQueue;
- (void)removeMovieItem:(NSMutableDictionary *)movieItem;
- (void)renameMovieItem:(NSMutableDictionary *)movieItem newName:(NSString *)newMovieName;
- (NSMutableDictionary *)item:(NSMutableDictionary *)searchItem isInArray:(NSArray *)array;

// Backup
- (NSArray *)backupItems;
- (NSArray *)backupSubItemsForIdentifier:(NSString *)backupIdentifier;
- (void)setSelectedBackupIdentifiers:(NSArray *)selectedIdentifiers;
- (void)restoreBackupWithIdentifier:(NSString *)theIdentifier;
- (void)deleteBackupWithIdentifier:(NSString *)theIdentifier;

@end
